package com.contextspace.objectdb_example.model;

import javax.jdo.annotations.Index;
import javax.persistence.Basic;
import javax.persistence.Entity;
import javax.persistence.Transient;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

@Entity
public class Attribute extends IDHolder {
	
    static Logger log = LoggerFactory.getLogger(Attribute.class.getName());
    
    private String name;
    private String value;
    private String type;
    private String displayName;
    private boolean encrypted = false;
    
    /**
     * Constructor 
     * @param name
     * @param values
     */
    public Attribute(String name, Object value) {
        this.name = name;
        this.type = value.getClass().getName();
        this.value = value.toString();
    }
    
    /**
     * Constructor 
     * @param nam
     */
    public Attribute(String name) {
        this.name = name;
    }
    
    /**
     * Default constructor
     */
    public Attribute() {
    }
    
    /**
     * @return the name
     */
    @Index
    @Basic
    public String getName() {
        return name;
    }
    /**
     * @param name the name to set
     */
    public void setName(String name) {
        this.name = name;
    }

    /**
     * @return the values
     */
    @Transient
    public Object getValue() {
        if(type.equals(String.class.getName())) {
            return value;
        } else if(type.equals(Boolean.class.getName())) {
            return new Boolean(value);
        } else if(type.equals(Integer.class.getName())) {
            return new Integer(value);
        } else if(type.equals(Long.class.getName())) {
            return new Long(value);
        } else {
            try {
                return Class.forName(type).getConstructor(String.class).newInstance(value);
            } catch (Exception e) {
                log.error("Failed to recreate object in attribute: " + e.getMessage(), e);
                return null;
            }
        }
    }


    /**
     * @param values the values to set
     */
    public void setValue(Object value) {
        if(value != null) {
            this.type = value.getClass().getName();
            this.value = value.toString();
        }
    }
    
    @Index
    @Basic
    public String getValueAsString() {
        return value;
    }
    
    public void setValueAsString(String value) {
        this.value = value;
    }

    /**
     * @return the displayName
     */
    public String getDisplayName() {
        return displayName;
    }

    /**
     * @param displayName the displayName to set
     */
    public void setDisplayName(String displayName) {
        this.displayName = displayName;
    }

    /**
     * @return the encrypted
     */
    public boolean isEncrypted() {
        return encrypted;
    }

    /**
     * @param encrypted the encrypted to set
     */
    public void setEncrypted(boolean encrypted) {
        this.encrypted = encrypted;
    }
    
    public String getType() {
        return type;
    }

    public void setType(String type) {
        this.type = type;
    }
    
    /* (non-Javadoc)
     * @see java.lang.Object#toString()
     */
    @Override
    public String toString() {
        StringBuffer buf = new StringBuffer();
        buf.append("<attribute name=\"").append(name).append("\" encrypted=\"").append(encrypted);
        buf.append("\" type=\"").append(type);
        buf.append("\" value=\"").append(value).append("\"");
        if(getId() != 0) {
            buf.append(" id=\"").append(getId()).append("\"");
        }
        if(displayName != null) {
            buf.append("\" displayName=\"").append(displayName);
        }
        buf.append("/>");
        return buf.toString();
    }
        
    @Override
    public boolean equals(Object obj) {
        if (obj == null || (getClass() != obj.getClass())) {
            return false;
        }
        if (this == obj) {
            return true;
        }
       
        Attribute other = (Attribute) obj;
        if (value == null || other.getValue() == null) {
            return false;                
        } else if ((name.equals(other.getName()) && value.equals(other.getValue()) || (getId() == other.getId()))) {
            return true;
        }
        return false;
    }
}
