package com.contextspace.objectdb_example.model;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

import javax.persistence.CascadeType;
import javax.persistence.Entity;
import javax.persistence.FetchType;
import javax.persistence.OneToMany;
import javax.xml.bind.annotation.XmlElement;
import javax.xml.bind.annotation.XmlElementWrapper;


@Entity
public class ContactDetails {

    private List<ElectronicAddress> electronicAddresses = new ArrayList<ElectronicAddress>();
    
    private List<Address> addresses = new ArrayList<Address>();
    
    private List<PhoneNumber> phoneNumbers = new ArrayList<PhoneNumber>();
    
    /**
     * Default Constructor
     */
    public ContactDetails() {
        
    }
    
    /**
     * @param emailAddresses
     * @param addresses
     * @param phoneNumbers
     */
    public ContactDetails(List<ElectronicAddress> emailAddresses, List<Address> addresses, List<PhoneNumber> phoneNumbers) {
        this.electronicAddresses = emailAddresses;
        this.addresses = addresses;
        this.phoneNumbers = phoneNumbers;
    }
    
    /**
     * @param electronicAddress
     */
    public void addElectronicAddress(ElectronicAddress electronicAddress) {
        electronicAddresses.add(electronicAddress);
    }

    /**
     * @return the electronicAddresses
     */
    @OneToMany(cascade=CascadeType.ALL, fetch=FetchType.EAGER)
    @XmlElementWrapper(name="electronicAddresses")
    @XmlElement(name = "electronicAddress")
    public List<ElectronicAddress> getElectronicAddresses() {
        return electronicAddresses;
    }

    /**
     * @param electronicAddresses the electronicAddresses to set
     */
    public void setElectronicAddresses(List<ElectronicAddress> electronicAddresses) {
        this.electronicAddresses = electronicAddresses;
    }
    
    /**
     * @param electronicAddressToRemove
     */
    public void removeElectronicAddress(ElectronicAddress electronicAddressToRemove) {
        electronicAddresses.remove(electronicAddressToRemove);
    }
    
    /**
     * @param electronicAddressToReplace
     */
    public void replaceElectronicAddress(ElectronicAddress electronicAddressToReplace) {
        for(int i=0; i<electronicAddresses.size(); i++) {
            if(electronicAddresses.get(i).equals(electronicAddressToReplace)) {
                electronicAddresses.set(i, electronicAddressToReplace);
            }
        }
    }

    /**
     * @param address
     */
    public void addAddress(Address address) {
        addresses.add(address);
    }
    
    /**
     * @return the addresses
     */
    @OneToMany(cascade=CascadeType.ALL, fetch=FetchType.EAGER)
    @XmlElementWrapper(name="addresses")
    @XmlElement(name = "address")
    public List<Address> getAddresses() {
        return addresses;
    }

    /**
     * @param addresses the addresses to set
     */
    public void setAddresses(List<Address> addresses) {
        this.addresses = addresses;
    }
    
    /**
     * @param addressToRemove
     */
    public void removeAddress(Address addressToRemove) {
        addresses.remove(addressToRemove);
    }
    
    /**
     * @param addressToReplace
     */
    public void replaceAddress(Address addressToReplace) {
        for(int i=0; i<addresses.size(); i++) {
            if(addresses.get(i).equals(addressToReplace)) {
                addresses.set(i, addressToReplace);
            }
        }
    }

    /**
     * @param phoneNumber
     */
    public void addPhoneNumber(PhoneNumber phoneNumber) {
        phoneNumbers.add(phoneNumber);
    }
    
    /**
     * @return the phoneNumbers
     */
    @OneToMany(cascade=CascadeType.ALL, fetch=FetchType.EAGER)
    @XmlElementWrapper(name="phoneNumbers")
    @XmlElement(name = "phoneNumber")
    public List<PhoneNumber> getPhoneNumbers() {
        return phoneNumbers;
    }

    /**
     * @param phoneNumbers the phoneNumbers to set
     */
    public void setPhoneNumbers(List<PhoneNumber> phoneNumbers) {
        this.phoneNumbers = phoneNumbers;
    }
    
    /**
     * @param phoneNumberToRemove
     */
    public void removePhoneNumber(PhoneNumber phoneNumberToRemove) {
        phoneNumbers.remove(phoneNumberToRemove);
    }
    
    /**
     * @param phoneNumberToReplace
     */
    public void replacePhoneNumber(PhoneNumber phoneNumberToReplace) {
        for(int i=0; i<phoneNumbers.size(); i++) {
            if(phoneNumbers.get(i).equals(phoneNumberToReplace)) {
                phoneNumbers.set(i, phoneNumberToReplace);
            }
        }
    }
    
    /* (non-Javadoc)
     * @see java.lang.Object#toString()
     */
    @Override
    public String toString() {
        StringBuffer buf = new StringBuffer();
        if(!electronicAddresses.isEmpty() || !addresses.isEmpty() || !phoneNumbers.isEmpty()) {
            buf.append("<contactDetails>");
            for(Iterator<ElectronicAddress> i = electronicAddresses.iterator(); i.hasNext();) {
                ElectronicAddress electronicAddress = i.next();
                buf.append(electronicAddress.toString());
            }
            for(Iterator<Address> i = addresses.iterator(); i.hasNext();) {
                Address address = i.next();
                buf.append(address.toString());
            }
            for(Iterator<PhoneNumber> i = phoneNumbers.iterator(); i.hasNext();) {
                PhoneNumber phoneNumber = i.next();
                buf.append(phoneNumber.toString());
            }
            buf.append("</contactDetails>");     
        }
        return buf.toString();
    }
}