package com.contextspace;

import java.io.File;
import java.util.Collection;
import java.util.List;

import javax.persistence.EntityManager;
import javax.persistence.EntityManagerFactory;
import javax.persistence.Persistence;
import javax.persistence.TypedQuery;
import javax.persistence.criteria.CriteriaBuilder;
import javax.persistence.criteria.CriteriaQuery;
import javax.persistence.criteria.Expression;
import javax.persistence.criteria.Join;
import javax.persistence.criteria.Predicate;
import javax.persistence.criteria.Root;

import static org.junit.Assert.*;

import org.junit.AfterClass;
import org.junit.BeforeClass;
import org.junit.Test;

import com.contextspace.objectdb_example.model.Address;
import com.contextspace.objectdb_example.model.Attribute;
import com.contextspace.objectdb_example.model.AttributeTypes;
import com.contextspace.objectdb_example.model.CDI;
import com.contextspace.objectdb_example.model.ContactDetails;
import com.contextspace.objectdb_example.model.ElectronicAddress;
import com.contextspace.objectdb_example.model.ElectronicAddress.ElectronicType;
import com.contextspace.objectdb_example.model.PhoneNumber;
import com.contextspace.objectdb_example.model.PhoneNumber.PhoneNumberTypes;

public class ExampleTest {
 
    private static EntityManagerFactory emf;
    private static EntityManager em;
    private static CDI createdCdi6;
    
    @BeforeClass
    public static void onceStart() throws Exception {
        emf = Persistence.createEntityManagerFactory("objectdb/db/test.odb");
        em = emf.createEntityManager();
 
        em.getTransaction().begin();
        
        Address address = createAddress("2000 Avenue Of The Stars", "Beverly Hills", "90067", "Los Angeles", "CA", "USA");
        PhoneNumber phoneNumber = createPhoneNumber(PhoneNumberTypes.Work, "+1", "310", "113 9999");
        ElectronicAddress electronicAddress = createElectronicAddress("abaldwin");
        CDI createdCdi1 = createCDI("abaldwin", "Alex", "Rae", "Baldwin", address, electronicAddress, phoneNumber);
        em.persist(createdCdi1);
        
        Address address2 = createAddress("550n S. Barrington Ave. #1214 ", "Westwood", "90049", "Los Angeles", "CA", "USA");
        PhoneNumber phoneNumber2 = createPhoneNumber(PhoneNumberTypes.Work, "+1", "310", "115 8888");
        ElectronicAddress electronicAddress2 = createElectronicAddress("sbaldwin");
        CDI createdCdi2 = createCDI("sbaldwin", "Stephen", "Andrew", "Baldwin", address2, electronicAddress2, phoneNumber2);
        em.persist(createdCdi2);
        
        Address address3 = createAddress("955 S. Carrillo Drive", "Beverly Hills", "90048-5400", "Los Angeles", "CA", "USA");
        PhoneNumber phoneNumber3 = createPhoneNumber(PhoneNumberTypes.Work, "+1", "323", "116 7777");
        ElectronicAddress electronicAddress3 = createElectronicAddress("wbaldwin");
        CDI createdCdi3 = createCDI("wbaldwin", "William", null, "Baldwin", address3, electronicAddress3, phoneNumber3);
        em.persist(createdCdi3);
        
        Address address4 = createAddress("PO Box 21", "Honiton", "EX14 1YH", "Honiton", "Devon", "UK");
        PhoneNumber phoneNumber4 = createPhoneNumber(PhoneNumberTypes.Work, "+44", "20", "8765 4321");
        ElectronicAddress electronicAddress4 = createElectronicAddress("kbeckinsale");
        CDI createdCdi4 = createCDI("kbeckinsale", "Kate", null, "Beckinsale", address4, electronicAddress4, phoneNumber4);
        em.persist(createdCdi4);
        
   
        
        em.getTransaction().commit();
        
    }
    
    @AfterClass
    public static void onceEnd() throws Exception {
        
        if(em != null) {
            em.close();
        }
        
        if(emf != null) {
            emf.close();
        }
        
        File f1 = new File("objectdb/db/test.odb");
        if(f1.exists()) {
            f1.delete();
        }
    }
    
    protected static CDI createCDI(String username, String first, String middle, String last, Address address, ElectronicAddress electronicAddress, PhoneNumber phoneNumber) {
        CDI cdi = new CDI();
        cdi.setCdiId(username);
        Attribute firstname = new Attribute(AttributeTypes.firstname, first);
        cdi.addAttribute(firstname);
        Attribute lastname = new Attribute(AttributeTypes.lastname, last);
        cdi.addAttribute(lastname);
        if(middle != null) {
            Attribute middlename = new Attribute(AttributeTypes.middlenames, middle);
            cdi.addAttribute(middlename);
        }
        Attribute title = new Attribute(AttributeTypes.title, "Mr");
        cdi.addAttribute(title);
        
        Attribute alias = new Attribute(AttributeTypes.alias, 10);
        cdi.addAttribute(alias);
        
        Attribute knownas = new Attribute(AttributeTypes.knownas, "5");
        cdi.addAttribute(knownas);
        
        ContactDetails contactDetails = new ContactDetails();
        contactDetails.addAddress(address);
        contactDetails.addElectronicAddress(electronicAddress);
        contactDetails.addPhoneNumber(phoneNumber);
        
        cdi.setContactDetails(contactDetails);
        
        return cdi;
    }
    
    protected static PhoneNumber createPhoneNumber(PhoneNumberTypes phoneType, String countrycode, String areacode, String localnumber) {
        PhoneNumber phoneNumber = new PhoneNumber();
        if(phoneType != null) {
            phoneNumber.setType(phoneType); 
        }
        else { 
            phoneNumber.setType(PhoneNumberTypes.Home);  
        }
        
        if(countrycode != null) { 
            Attribute countrycodeAttr = new Attribute(AttributeTypes.countrycode, countrycode);
            phoneNumber.addAttribute(countrycodeAttr);
        }
        else {
            Attribute countrycodeAttr = new Attribute(AttributeTypes.countrycode, "+61");
            phoneNumber.addAttribute(countrycodeAttr);
        }
        
        if(areacode != null) {
            Attribute areacodeAttr = new Attribute(AttributeTypes.areacode, areacode);
            phoneNumber.addAttribute(areacodeAttr);
        }
        else {
            Attribute areacodeAttr = new Attribute(AttributeTypes.areacode, "2");
            phoneNumber.addAttribute(areacodeAttr);
        }
        
        if(localnumber != null) {
            Attribute localnumberAttr = new Attribute(AttributeTypes.localnumber, localnumber);
            phoneNumber.addAttribute(localnumberAttr);
        }
        else {
            Attribute localnumberAttr = new Attribute(AttributeTypes.localnumber, "99999999");
            phoneNumber.addAttribute(localnumberAttr);
        }
        
        return phoneNumber;
    }
    
    protected static ElectronicAddress createElectronicAddress(String username) {
        ElectronicAddress electronicAddress = new ElectronicAddress(ElectronicType.Email, username + "@test.com");
        return electronicAddress;
    }
    
    protected static Address createAddress(String addressLine, String suburb, String postalCode, String city, String state, String country){
        //ADDRESS
        Address address = new Address(true);
        
        if(addressLine != null) {
            Attribute addressLine1Attr = new Attribute(AttributeTypes.addressline, addressLine);
            address.addAttribute(addressLine1Attr);
        }
        else {
            Attribute addressLine1Attr = new Attribute(AttributeTypes.addressline, "Unit 12");
            Attribute addressLine2Attr = new Attribute(AttributeTypes.addressline, "Pacific Hwy");
            address.addAttribute(addressLine1Attr);
            address.addAttribute(addressLine2Attr);
        }
        
        if(suburb != null) {
            Attribute suburbAttr = new Attribute(AttributeTypes.suburb, suburb);
            address.addAttribute(suburbAttr);
        }
        else {
            Attribute suburbAttr = new Attribute(AttributeTypes.suburb, "Lane Cove North");
            address.addAttribute(suburbAttr);
        }
        
        if(postalCode != null) {
            Attribute postalcodeAttr = new Attribute(AttributeTypes.postalcode, postalCode);
            address.addAttribute(postalcodeAttr);
        }
        else {
            Attribute postalcodeAttr = new Attribute(AttributeTypes.postalcode, "2066");
            address.addAttribute(postalcodeAttr);
        }
        
        if(state != null){
            Attribute stateAttr = new Attribute(AttributeTypes.state, state);
            address.addAttribute(stateAttr);
        }
        else {
            Attribute stateAttr = new Attribute(AttributeTypes.state, "NSW");
            address.addAttribute(stateAttr);
        }
        
        if(city != null) {
            Attribute cityAttr = new Attribute(AttributeTypes.city, city);
            address.addAttribute(cityAttr);
        }
        else {
            Attribute cityAttr = new Attribute(AttributeTypes.city, "Sydney");
            address.addAttribute(cityAttr);
        }
        
        if(country != null) {
            Attribute countryAttr = new Attribute(AttributeTypes.country, country);
            address.addAttribute(countryAttr);
        }
        else {
            Attribute countryAttr = new Attribute(AttributeTypes.country, "Australia");
            address.addAttribute(countryAttr);
        } 
        
        return address;
    }
    
    
    @Test
    public void queryAttribute1() throws Exception {
        
        CriteriaBuilder cb = em.getCriteriaBuilder();
        CriteriaQuery<CDI> q = cb.createQuery(CDI.class);
        Root<CDI> cdi = q.from(CDI.class);
        
        Join<CDI, Attribute> attributeJoin = cdi.join("attributeList");
        Predicate pred = 
                cb.and(
                    cb.equal(attributeJoin.get("valueAsString"), "Alex"), 
                    cb.equal(attributeJoin.get("name"), "firstname")
                );
        
        q.select(cdi).where(pred);
        TypedQuery<CDI> typedQuery = em.createQuery(q);
        List<CDI> results = typedQuery.getResultList();
        System.out.println("RESULT: " + results);
        assertEquals(1, results.size());
        assertEquals("Alex", results.get(0).getAttribute("firstname").get(0).getValue());
    }
    
    
    @Test
    public void queryAttribute2() throws Exception {
        
        CriteriaBuilder cb = em.getCriteriaBuilder();
        CriteriaQuery<CDI> q = cb.createQuery(CDI.class);
        Root<CDI> cdi = q.from(CDI.class);
        
        Join<CDI, Attribute> attributeJoin = cdi.join("attributeList");
        Join<CDI, Attribute> attributeJoin2 = cdi.join("attributeList");
        Predicate pred = 
            cb.and(
                cb.and(
                    cb.equal(attributeJoin.get("valueAsString"), "Alex"), 
                    cb.equal(attributeJoin.get("name"), "firstname")
                ),
                cb.and(
                    cb.equal(attributeJoin2.get("valueAsString"), "Baldwin"), 
                    cb.equal(attributeJoin2.get("name"), "lastname")
                )
            );
        
        q.select(cdi).where(pred);
        TypedQuery<CDI> typedQuery = em.createQuery(q);
        List<CDI> results = typedQuery.getResultList();
        System.out.println("RESULT: " + results);
        assertEquals(1, results.size());
        assertEquals("Alex", results.get(0).getAttribute("firstname").get(0).getValue());
    }
    
    @Test
    public void queryAttribute3() throws Exception {
        
        CriteriaBuilder cb = em.getCriteriaBuilder();
        CriteriaQuery<CDI> q = cb.createQuery(CDI.class);
        Root<CDI> cdi = q.from(CDI.class);
        
        Join<CDI, Attribute> attributeJoin = cdi.join("attributeList");
        Predicate pred = 
            cb.or(
                cb.and(
                    cb.equal(attributeJoin.get("valueAsString"), "Alex"), 
                    cb.equal(attributeJoin.get("name"), "firstname")
                ),
                cb.and(
                    cb.equal(attributeJoin.get("valueAsString"), "William"), 
                    cb.equal(attributeJoin.get("name"), "firstname")
                )
            );
        
        q.select(cdi).where(pred);
        System.out.println("QUERY: " + q.toString());
        TypedQuery<CDI> typedQuery = em.createQuery(q);
        List<CDI> results = typedQuery.getResultList();
        System.out.println("RESULT: " + results);
        assertEquals(2, results.size());
    }
    
    @Test
    public void queryAttribute3a() throws Exception {
        
        CriteriaBuilder cb = em.getCriteriaBuilder();
        CriteriaQuery<CDI> q = cb.createQuery(CDI.class);
        Root<CDI> cdi = q.from(CDI.class);
        
        Join<CDI, Attribute> attributeJoin = cdi.join("attributeList");
        
        Predicate pred1 = cb.equal(attributeJoin.get("name"), "firstname");
        Predicate pred2 = cb.equal(attributeJoin.get("valueAsString"), "Alex");
        Predicate pred3 = cb.and(pred1, pred2);
        
        Predicate pred4 = cb.equal(attributeJoin.get("valueAsString"), "William"); 
        Predicate pred5 = cb.equal(attributeJoin.get("name"), "firstname");
        Predicate pred6 = cb.and(pred4, pred5);
        
        Predicate pred7 = cb.or(pred3,pred6);
        
        
        /*Predicate pred = 
            cb.or(
                cb.and(
                    cb.equal(attributeJoin.get("valueAsString"), "Alex"), 
                    cb.equal(attributeJoin.get("name"), "firstname")
                ),
                cb.and(
                    cb.equal(attributeJoin.get("valueAsString"), "William"), 
                    cb.equal(attributeJoin.get("name"), "firstname")
                )
            );
        */
        q.select(cdi).where(pred7);
        System.out.println("QUERY: " + q.toString());
        TypedQuery<CDI> typedQuery = em.createQuery(q);
        List<CDI> results = typedQuery.getResultList();
        System.out.println("RESULT: " + results);
        assertEquals(2, results.size());
    }
    
    @Test
    public void queryAttribute4() throws Exception {
        
        TypedQuery<CDI> query = em.createQuery("SELECT DISTINCT t FROM com.contextspace.objectdb_example.model.CDI t JOIN t.attributeList attr JOIN t.attributeList attr2 WHERE " +
                "attr.valueAsString='Alex' and attr.name='firstname' and attr2.valueAsString='Baldwin' and attr2.name='lastname'", CDI.class);

        List<CDI> results = query.getResultList();
        System.out.println("RESULT: " + results);
        assertEquals(1, results.size());
    }
    
    @Test
    public void queryAttribute5() throws Exception {
        
        TypedQuery<CDI> query = em.createQuery("SELECT DISTINCT t FROM com.contextspace.objectdb_example.model.CDI t JOIN t.attributeList attr WHERE " +
                "(attr.valueAsString='Alex' and attr.name='firstname') or (attr.valueAsString='William' and attr.name='firstname')", CDI.class);

        List<CDI> results = query.getResultList();
        System.out.println("RESULT: " + results);
        assertEquals(2, results.size());
    }
    
    @Test
    public void queryAttribute6() throws Exception {
        
        TypedQuery<CDI> query = em.createQuery("SELECT DISTINCT t FROM com.contextspace.objectdb_example.model.CDI t JOIN t.attributeList attr WHERE " +
                "attr.valueAsString='5' and attr.name='knownas'", CDI.class);

        List<CDI> results = query.getResultList();
        System.out.println("RESULT: " + results);
        assertEquals(4, results.size());
    }
    
    @Test
    public void queryAttribute7() throws Exception {
        
        TypedQuery<CDI> query = em.createQuery("SELECT DISTINCT t FROM com.contextspace.objectdb_example.model.CDI t JOIN t.attributeList attr WHERE " +
                "attr.valueAsString='10' and attr.name='alias'", CDI.class);

        List<CDI> results = query.getResultList();
        System.out.println("RESULT: " + results);
        assertEquals(4, results.size());
    }
    
    @Test
    public void queryAttribute8() throws Exception {
        TypedQuery<CDI> query = em.createQuery("SELECT DISTINCT $1 FROM CDI $1 JOIN $1.attributeList $2 JOIN $1.attributeList $3 " +
                    "WHERE ($2.name='firstname') AND ($2.valueAsString='Alex') AND ($3.name='lastname') AND ($3.valueAsString='Baldwin')", CDI.class);
        
        List<CDI> results = query.getResultList();
        System.out.println("RESULT: " + results);
        assertEquals(1, results.size());
    }
    
    @Test
    public void queryAttribute9() throws Exception {
        TypedQuery<CDI> query = em.createQuery("SELECT $1 FROM CDI $1 JOIN $1.attributeList $2 JOIN $1.attributeList $3 " +
                    "WHERE ($2.name='firstname') AND ($2.valueAsString='Alex') AND ($3.name='lastname') AND ($3.valueAsString='Baldwin')", CDI.class);
        
        List<CDI> results = query.getResultList();
        System.out.println("RESULT: " + results);
        assertEquals(1, results.size());
    }
    
    @Test
    public void queryAttribute10() throws Exception {
        
        CriteriaBuilder cb = em.getCriteriaBuilder();
        CriteriaQuery<CDI> q = cb.createQuery(CDI.class);
        Root<CDI> cdi = q.from(CDI.class);
        
        Predicate pred7 = cb.equal(cdi.get("cdiId"),"abaldwin");
        
        q.distinct(true);
        q.select(cdi).where(pred7);
        System.out.println("QUERY: " + q.toString());
        TypedQuery<CDI> typedQuery = em.createQuery(q);
        List<CDI> results = typedQuery.getResultList();
        System.out.println("RESULT: " + results);
        assertEquals(1, results.size());
        
    }
    
    @Test
    public void queryAttribute11() throws Exception {
        TypedQuery<CDI> query = em.createQuery("SELECT DISTINCT $1 FROM CDI $1 JOIN $1.contactDetails $2 JOIN $2.phoneNumbers $3 JOIN $3.attributeList $4 " +
                "WHERE (($4.name='areacode') AND ($4.valueAsString='310'))", CDI.class);
        
        List<CDI> results = query.getResultList();
        System.out.println("RESULT: " + results);
        assertEquals(2, results.size());
    }
    
    @Test
    public void queryAttribute12() throws Exception {
        TypedQuery<CDI> query = em.createQuery("SELECT DISTINCT $1 FROM CDI $1 JOIN $1.contactDetails $2 JOIN $2.addresses $3 JOIN $3.attributeList $4 " +
        		"WHERE (($4.name='suburb') AND ($4.valueAsString='Westwood'))", CDI.class);
        
        List<CDI> results = query.getResultList();
        System.out.println("RESULT: " + results);
        assertEquals(1, results.size());
    }
    
    @Test
    public void queryAttribute13() throws Exception {
        TypedQuery<CDI> query = em.createQuery("SELECT DISTINCT $1 FROM CDI $1 JOIN $1.contactDetails $2 JOIN $2.addresses $3 JOIN $3.attributeList $4 " +
                "JOIN $2.phoneNumbers $5 JOIN $5.attributeList $6 " +
                "WHERE ((($4.name='suburb') AND ($4.valueAsString='Los Angeles')) AND (($6.name='areacode') AND ($6.valueAsString='310')))", CDI.class);
        
        List<CDI> results = query.getResultList();
        System.out.println("RESULT: " + results);
        assertEquals(2, results.size());
    }
    
    
        
}
